

#include "stm32f0xx_conf.h"
#include "lcd.h"
#include "delay.h"
#include "lib_i2c.h"

//#define LCD_PWM_BACKLIGHT

#define PIN_RS       (1 << 0)
#define PIN_EN       (1 << 2)
#define BACKLIGHT    (1 << 3)

static void LCD_SendInternal(uint8_t lcd_addr, uint8_t data, uint8_t flags);
static void LCD_SendCommand(uint8_t lcd_addr, uint8_t cmd);
static void LCD_SendData(uint8_t lcd_addr, uint8_t data);


#define US_DELAY1     1000
#define US_DELAY2     500
static void LCD_SendInternal(uint8_t lcd_addr, uint8_t data, uint8_t flags)
{
   uint8_t up = data & 0xF0;
   uint8_t lo = (data << 4) & 0xF0;

   uint8_t data_arr[4];
   data_arr[0] = up|flags|BACKLIGHT|PIN_EN;
   data_arr[1] = up|flags|BACKLIGHT;
   data_arr[2] = lo|flags|BACKLIGHT|PIN_EN;
   data_arr[3] = lo|flags|BACKLIGHT;

   I2C_ByteWrite(I2C1, LCD_ADDR, data_arr[0]);
   us_delay(US_DELAY1);
   I2C_ByteWrite(I2C1, LCD_ADDR, data_arr[1]);
   us_delay(US_DELAY2);
   I2C_ByteWrite(I2C1, LCD_ADDR, data_arr[2]);
   us_delay(US_DELAY1);
   I2C_ByteWrite(I2C1, LCD_ADDR, data_arr[3]);
   us_delay(US_DELAY2);
}


static void LCD_SendCommand(uint8_t lcd_addr, uint8_t cmd)
{
   LCD_SendInternal(lcd_addr, cmd, 0);
}

static void LCD_SendData(uint8_t lcd_addr, uint8_t data)
{
   LCD_SendInternal(lcd_addr, data, PIN_RS);
}



void LCD_Init(uint8_t lcd_addr)
{
   Init_I2C();
   DelayMSec(5);

   // 4-bit mode, 2 lines, 5x7 format
   LCD_SendCommand(lcd_addr, 0b00110000);
   // display & cursor home (keep this!)
   LCD_SendCommand(lcd_addr, 0b00000010);
   // display on, right shift, underline off, blink off
   LCD_SendCommand(lcd_addr, 0b00001100);
   // clear display (optional here)
   LCD_SendCommand(lcd_addr, 0b00000001);
   DelayMSec(20);
}

void LCD_HomeCursor(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_RETURNHOME);  // Home Cursor.
   DelayMSec(5);
}

void LCD_ClearDisplay(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_CLEARDISPLAY);
   DelayMSec(5);
}

void LCD_SetDDRAMAddr(unsigned char addr)
{
   addr &= 0x7F; // only 7 bits are valid
   LCD_SendCommand(LCD_ADDR, LCD_SETDDRAMADDR | addr);
}


// Line 1 start address is 0, Line 2 address starts from 0x40
void LCD_GotoLine2(void)
{
   LCD_SetDDRAMAddr(0x40);
   DelayMSec(3);
}


void LCD_setCursor(uint8_t col, uint8_t row)
{
   int row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };
   if ( row > LCD_ROW )
   {
      row = LCD_ROW - 1;    // we count rows starting w/0
   }
   LCD_SendCommand(LCD_ADDR, LCD_SETDDRAMADDR | (col + row_offsets[row]));
   DelayMSec(5);
}


void LCD_PrintStr(unsigned char *str)
{
   unsigned char *p = str;
   while (*p)
   {
      LCD_SendData(LCD_ADDR, *p++);
   }
}


// Each call scrolls left 1 char
void LCD_ScrollLeft(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}

// Each call scrolls right 1 char
void LCD_ScrollRight(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}


void LCD_EntryModeDispShift(unsigned char on)
{
   LCD_SendCommand(LCD_ADDR, LCD_ENTRYMODESET | LCD_ENTRYLEFT | (on & 0x01)); // shift is bit 0.
}


// Turn on and off the blinking cursor
void LCD_NoBlink(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_DISPLAYCONTROL | LCD_DISPLAYON);
}

void LCD_Blink(void)
{
   LCD_SendCommand(LCD_ADDR, LCD_DISPLAYCONTROL | LCD_DISPLAYON | LCD_CURSORON | LCD_BLINKON);
}


// write to STM32F030 I2C Slave that controls LCD backlight..
// slave address is 0x23.
void LCD_PWMBacklight(uint8_t pwm)
{
#ifdef LCD_PWM_BACKLIGHT
   uint8_t dsBuf[4];

   dsBuf[0] = dsBuf[1] = dsBuf[3] = 0;
   dsBuf[2] = pwm;

   I2C_WrReg((0x23 << 1), 0, dsBuf, 4);
#endif
}






